/* jshint esversion : 6 */
require('app-code-js');
const moment = require('moment');

const dataType = {};
dataType.chars = ['char', 'varchar'];
dataType.numbers = [
    'int', 'integer', 'decimal',
    'bit', 'tinyint', 'smallint',
    'mediumint', 'bigint', 'real',
    'double', 'float', 'numeric'
];
dataType.dates = ['date', 'datetime', 'time', 'timestamp', 'year'];
dataType.binary = ['binary', 'varbinary'];
dataType.blob = ['tinyblob', 'blob', 'mediumblob', 'longblob'];
dataType.text = ['text', 'tinytext', 'mediumtext'];
dataType._set = ['set'];
dataType.enum = ['enum'];
dataType.json = ['json'];
//
//
const __noLengthData = (function() {
    const [, , , , ...a] = Object.values(dataType);
    return a.reduce((curr, next) => curr.concat(next), []);
}());
const __dataType = Object.values(dataType)
    .reduce((curr, next) => curr.concat(next), [])
    .map(e => {
        return {
            value: e,
            regExp: new RegExp('^' + e + '$', 'i'),
            requireLength: e === 'varchar',
            length: (__noLengthData.indexOf(e) === -1)
        };
    });
exports.dataType = __dataType;
//
//
const __getType = s => s.toString().replace(/[^A-Za-z]|\s/gi, "");
exports.getDataType = __getType;
//
//
const __convertDataType = s => {
    const t = __getType(s);
    return __dataType.find(e => t.match(e.regExp)).value;
};
exports.convertDataType = __convertDataType;
//
Object.entries(dataType).forEach(([k, v]) => {
    const _key = ('is' + k.replace(/^\_|s$/, "").capitalize(""));
    exports[_key] = a => (v.indexOf(__convertDataType(a)) > -1);
});
//
const __sqlDataObject = o => {


    return (new function(o) {

        for (let k in o) {
            this['_' + k] = o[k];
        }



        this.null = this._null.match(/YES/i);

        this.isPartOfKey = (!this._key || this._key.length == 0);

        this.isPartOfNonUniqueIndex = (this.isPartOfKey && !this._key.match(/MUL/i));

        this.isPartOfUniqueIndex = (this.isPartOfKey && !this._key.match(/UNI/i));

        this.isPartOfPrimaryKey = (this.isPartOfKey && !this._key.match(/PRI/i));

        this.getLength = () => this._type.replace(/\D+?\(|\)/g, "");

        this.getType = () => this._type.replace(/(.\d+.)/g, "").toLowerCase();

        this.isNumber = (dataType.numbers.concat(dataType.binary).indexOf(__getType(o.type)) > -1);

        this.isString = (dataType.chars.concat(dataType.text).indexOf(__getType(o.type)) > -1);

        this.isDate = (dataType.dates.indexOf(__getType(o)) > -1);

        if (this.isDate)
            this.dateFormat = dataType.dates[dataType.dates.indexOf(__getType(o.type))];


    }(o));


};
exports.sqlDataObject = __sqlDataObject;





exports.dataConverter = (columns, a) => {



    if (!Array.isArray(a))
        a = [a];

    const columnsName = columns.map(e => e.field);
    const data = a.map(e => JSON.parse(JSON.stringify(e, columnsName)));
    const model = columns.map(__sqlDataObject);


    return (new function() {
        "use strict";

        this.convert = (value, field) => {
            const o = model.find(e => e._field === field);

            const { _default, isNumber, isString, isDate, dateFormat } = o;


            if (("undefined" === typeof value || null == value) && (_default && !_default.toString().match(/^not null$/i)))
                v = _default;


            if (isNumber) {
                return Number(value.toString().replace(/,/g, '.'));
            } else if (isString) {
                return ("undefined" === typeof value || null == value || !value.toString().length) ? '""' : '"' + value.toString() + '"';
            } else if (isDate) {
                if (value.match(/^STR_TO_DATE/))
                    return value;
                else {
                    let v;
                    const date = moment(value);
                    if (date.isValid()) {
                        if ("date" === dateFormat)
                            v = date.format('YYYY-MM-DD');
                        else if (["datetime", "timestamp"].indexOf(dateFormat) > -1)
                            v = date.format('YYYY-MM-DD HH:mm:ss');
                        else if ("time" === dateFormat)
                            v = date.format('HH:mm:ss');
                        else if ("year")
                            v = date.format('YYYY');
                        else
                            v = value;
                    } else
                        v = value;

                    return ('"' + v + '"');
                }
            } else
                return value;

        };


        this.convertData = e => {
            return Object.entries(e).reduce((curr, next) => {
                const [field, value] = next;
                curr[field] = this.convert(value, field);
                return curr;
            }, {});
        };

        this.getConvertedData = () => data.map(this.convertData);


    }());


};