/* jshint esversion : 6 */

require('app-code-js');

const { setColumnLength, setColumnType, getColumnType, setColumnDefault } = require('./constaint');
const setColumn = require('./cmp');



class MySqlColumns_Query extends require('../db-foreign-key/index') {
    constructor(opt) {
        super(opt);
    }

    showColumns(tableName, dbName) {
        return this.query(`SHOW columns FROM ${this.db_dot_table(tableName, dbName)}`);
    }

    /**
     * @param {string} tableName "name of table" 
     * @param {Object.<{name:string, type:string, default:string, length:(string|number)}>} columns
     * @param {string} [dbName] "name of db"
     */
    addColumns(tableName, columns, dbName) {
        const query = `ALTER TABLE ${this.db_dot_table(tableName, dbName)} ${this.addColumnsQuery(columns)}`;
        return this.query(query);
    }
    addColumnsQuery(columns) {
        return columns.map(e => `ADD COLUMN ${setColumn(e, false)} `).join();
    }


    /**
     * @param {string} tableName "name of table" 
     * @param {string[]} columns "list of columns to drop"
     * @param {string} [dbName] "name of db"
     */
    dropColumns(tableName, columns, dbName) {
        const query = `ALTER TABLE ${this.db_dot_table(tableName, dbName)} ${this.dropColumnsQuery(columns)}`;
        return this.query(query);
    }
    dropColumnsQuery(columns) {
        return columns.map(e => `DROP COLUMN ${e} `).join();
    }



    /**
     * @param {string} tableName "name of table" 
     * @param {Object.<{name:string, type:string, default:string, length:(string|number)}>} columns
     * @param {string} [dbName] "name of db"
     */
    modifyColumns(tableName, columns, dbName) {
        const query = `ALTER TABLE ${this.db_dot_table(tableName, dbName)} ${this.modifyColumnsQuery(columns)}`;
        return this.query(query);
    }
    modifyColumnsQuery(columns) {
        return columns.map(e => `MODIFY COLUMN ${setColumn(e, true)} `).join();
    }
}


class MySqlColumns extends MySqlColumns_Query {
    constructor(opt) {
        super(opt);
    }

    getColumnsOptions(tableName, opt, dbName) {
        const q = $q.defer();



        this.showColumns(tableName, dbName).then(
            fields => {
                this.showIndex(tableName, { db: dbName }).then(
                    indexes => {

                        /* colonne che compongono foreign keys*/
                        const _referencedIndexes = indexes.filter(e => {
                            if (e.referenced_table_name === tableName)
                                return e;
                        }).map(e => e.referenced_column_name).unique();


                        /*  */
                        const __columns = fields.map(e => {
                            e.toLowerKeys();
                            e.name = e.field;
                            e.length = setColumnLength(e.type);
                            e.null = (e.null.match(/^yes$/i) ? true : false);
                            e.default = setColumnDefault(e);
                            if (e.key.match(/^pri$/i) || e.extra.match(/^auto_increment$/i)) {
                                e.autoincrement = null;
                                e.default = null;
                            }
                            e.type = e.type.replace(/\(+[0-9]+\)$/, "");
                            delete e.field;
                            return e;
                        });
                        const __priKeyFields = __columns.filter(e => {
                            if (e.key.match(/^pri$/i) || e.extra.match(/^auto_increment$/i))
                                return e;
                        }).map(e => e.name);


                        const create = [];
                        const alter = [];


                        const checkColumns = o => {

                            if (__priKeyFields.indexOf(o.name) > -1)
                                return;

                            const col = __columns.find(e => e.name === o.name);


                            if (!col && _referencedIndexes.indexOf(o.name) === -1)
                                create.push(o);
                            else if (col) {

                                if (__priKeyFields.indexOf(o.name) > -1) {
                                    ['autoincrement', 'default'].forEach(k => o[k] = null);
                                } else if (__priKeyFields.indexOf(o.name) === -1 && __priKeyFields.indexOf(col.name) === -1) {
                                    delete o.autoincrement;
                                }




                                if (!o.hasOwnProperty('null'))
                                    o.null = col.null;

                                o.default = setColumnDefault(o);


                                if (!o.hasOwnProperty('length'))
                                    o.length = col.length;



                                const __checkO = {};
                                __checkO.completeAssign(o);
                                delete __checkO.length;
                                const __checkCol = JSON.parse(JSON.stringify(col, Object.keys(__checkO)));


                                if (JSON.stringify(__checkO) !== JSON.stringify(__checkCol))
                                    alter.push(o);

                            }

                        };

                        opt.forEach(v => checkColumns(v));


                        q.resolve({
                            alter: alter,
                            create: create,
                            drop: __columns.filter(e => {
                                if (!opt.find(o => o.name === e.name))
                                    return e.name;
                            }).map(e => e.name)
                        });

                    },
                    q.reject
                );


            },
            q.reject
        );


        return q.promise;


    }


    columns(tableName, opt, dbName) {
        const q = $q.defer();

        this.getColumnsOptions(tableName, opt, dbName).then(
            r => {

                const queue = $q.chain();

                const execFn = (a, cmd) => () => {
                    const q = $q.defer();
                    if (!a.length)
                        q.resolve();
                    else {
                        this[cmd](tableName, a, dbName) - then(q.resolve, q.reject);
                    }

                    return q.promise;
                };


                const _qChain = $q.chain(),
                    _queue = [
                        execFn(r.drop, 'dropColumns'),
                        execFn(r.alter, 'modifyColumns'),
                        execFn(r.create, 'addColumns')
                    ];

                _qChain.push.apply(_qChain, _queue).exec().then(q.resolve, q.reject);
            },
            q.reject
        );


        return q.promise;

    }
}


module.exports = MySqlColumns;