/* jshint esversion : 6 */

const mysql = require('mysql');
require('app-code-js');



function MySql(opt) {

    let ready = $q.defer();

    const { connectionType = "connection", connectionMode = "shared" } = (opt || {});
    const connectionParams = require('./connectionParams')(opt);
    connectionParams.connectionType = connectionType;
    connectionParams.connectionMode = connectionMode;



    let _Connection = null;


    class MySqlConnection {

        constructor(opt) {
            this._connectionParams = connectionParams;
        }

        get _connection() {
            return _Connection;
        }

        get ready() {
            return ready.promise;
        }

        get closed() {
            return (_Connection ? _Connection._closed : true);
        }
        connect(poolOpt) {
            // = $q.defer();
            const opt = connectionParams;


            if (this.closed) {
                testConnection().then(
                    ({ dbms, version }) => {

                        this.dbms = dbms;
                        this.version = version;


                        const connectionCmd = "connection" === connectionType ? "createConnection" : "createPool";
                        _Connection = mysql[connectionCmd](connectionParams);

                        if (connectionType === "connect") {
                            _Connection.connect();
                        }

                        ready.resolve(_Connection);
                    },
                    err => ready.reject(_Connection)
                );
            } else
                ready.resolve(_Connection);

            return this.ready;
        }


        destroyConnection(connection) {
            const q = $q.defer();
            //
            $q.when(this.ready, () => {
                if (!this.closed) {
                    _Connection.destroy();
                    ready = $q.defer();
                    q.resolve();
                } else
                    q.resolve();
            });

            return q.promise;
        }


    }

    /**
     * test connessione
     * se il database non esiste viene creato
     */
    function testConnection() {
        const q = $q.defer();

        if (!connectionParams.database)
            return q.resolve();
        else {

            const { database: dbName, ...testParams } = connectionParams;
            testParams.multipleStatements = true;
            const conn = mysql.createConnection(testParams);
            conn.connect();

            conn.query(`SELECT VERSION(); CREATE DATABASE IF NOT EXISTS ${dbName};`, (err, res, fields) => {
                if (err)
                    return q.reject(err);


                const [versionInfo, ] = res;
                const [version, dbms] = Object.values(versionInfo[0])[0].split(/\-/g);

                conn.destroy();
                q.resolve({ dbms: dbms.toLowerCase(), version: version });
            });

        }
        return q.promise;
    }



    const conn = new MySqlConnection();
    conn.connect();
    return conn;
}



module.exports = opt => MySql(opt);