/* jshint esversion : 6 */

module.exports = opt => {
    return {
        // The hostname of the database you are connecting to. (Default: localhost)
        "host": opt.host || "localhost",

        // The port number to connect to. (Default: 3306)        
        "port": opt.port || 3306,

        // The source IP address to use for TCP connection. (Optional)
        "localAddress": opt.localAddress || null,

        // The path to a unix domain socket to connect to. When used host and port are ignored
        "sockethPath": opt.sockethPath || null,

        // The MySQL user to authenticate as
        "user": opt.user || null,

        // The password of that MySQL user
        "password": opt.password || null,

        // Name of the database to use for this connection (Optional)
        "database": opt.database || null,

        //The charset for the connection. This is called "collation" in the SQL-level of MySQL (like utf8_general_ci). 
        //If a SQL-level charset is specified (like utf8mb4) then the default collation for that charset is used. (Default: 'UTF8_GENERAL_CI')
        "charset": opt.charset || "UTF8_GENERAL_CI",

        //The timezone configured on the MySQL server. This is used to type cast server date/time values to JavaScript Date object and vice versa. 
        //This can be 'local', 'Z', or an offset in the form +HH:MM or -HH:MM. (Default: 'local')
        "timezone": opt.timezone || "local",

        // The milliseconds before a timeout occurs during the initial connection to the MySQL server. (Default: 10000)
        "connectTimeout": opt.connectTimeout || 10000,

        // Stringify objects instead of converting to values. (Default: false)
        "stringifyObjects": opt.stringifyObjects || false,

        // Allow connecting to MySQL instances that ask for the old (insecure) authentication method. (Default: false)
        "insecureAuth": opt.insecureAuth || false,

        // Determines if column values should be converted to native JavaScript types. (Default: true)
        "typeCast": opt.typeCast || true,

        // A custom query format function. See https://github.com/mysqljs/mysql#custom-format
        "queryFormat": opt.queryFormat || null,

        // When dealing with big numbers (BIGINT and DECIMAL columns) in the database, you should enable this option (Default: false)
        "supportBigNumbers": opt.supportBigNumbers || false,

        // Enabling both supportBigNumbers and bigNumberStrings forces big numbers (BIGINT and DECIMAL columns) to be always returned as JavaScript String objects (Default: false). 
        // Enabling supportBigNumbers but leaving bigNumberStrings disabled will return big numbers as String objects only when they cannot be accurately represented with [JavaScript Number objects] (http://ecma262-5.com/ELS5_HTML.htm#Section_8.5) (which happens when they exceed the [-2^53, +2^53] range), otherwise they will be returned as Number objects. 
        // This option is ignored if supportBigNumbers is disabled
        "bigNumberStrings": opt.supportBigNumbers && opt.bigNumberStrings ? opt.bigNumberStrings : false,

        // Force date types (TIMESTAMP, DATETIME, DATE) to be returned as strings rather then inflated into JavaScript Date objects. 
        // Can be true/false or an array of type names to keep as strings. (Default: false)
        "dateStrings": opt.dateStrings || false,

        // Prints protocol details to stdout. Can be true/false or an array of packet type names that should be printed. (Default: false)
        "debug": opt.debug || false,

        // Allow multiple mysql statements per query. Be careful with this, it could increase the scope of SQL injection attacks. (Default: false)
        "multipleStatements": opt.multipleStatements || false,

        // List of connection flags to use other than the default ones. It is also possible to blacklist default ones. For more information, check https://github.com/mysqljs/mysql#connection-flags.
        "flags": setFlags(opt.flags),

        // object with ssl parameters or a string containing name of ssl profile. See https://github.com/mysqljs/mysql#ssl-options
        "ssl": opt.ssl || null,

        // connection limit for connection pool
        "connectionLimit": opt.connectionLimit
    };
};


/**
 * @param {(string|string[]|object)} o "if you pass an object it'll must have at least one of the keys ['blakList', 'whiteList'] and their value must be string[]"
 * @return {(string|null)}
 */
function setFlags(o) {
    if (!o)
        return null;
    if ("string" === typeof o)
        return o;
    else if (Array.isArray(O))
        return o.join();
    else if ("object" === typeof o) {
        let r = "";

        /**
         * @param {string[]} a 
         * @param {boolean} n "if setted set flags in the list to be deleted" 
         */
        const setR = (a, n) => a.map(e => (n ? '-' : '+') + e).join();


        if (!o.blackList && !o.whiteList)
            return null;
        else {
            const x = [];

            for (let k in o)
                x.push(setR(o[k], (k === 'blackList')));

            return x.join();
        }
    }
}