/* jshint esversion : 6 */

const { areEquals, typeOf } = require('app-code-js');
const __tranformToArray = a => ("array" === typeOf(a) ? a : [a]);


const _set = opt => {

    const exec = ({ name, columns, parent, parentColumns, onDelete, onUpdate }) => {

        const __columns = __tranformToArray(columns);
        const __parentColumns = __tranformToArray(parentColumns);


        if (__columns.length === __parentColumns.length) {

            const __setKeyColumns = c => (Array.isArray(c) ? c.join() : c);


            let r = ` CONSTRAINT ${name} `;
            // 
            // -- foreign key name
            r += ` FOREIGN KEY ${name} (${ __setKeyColumns(columns)})`;
            //
            // -- foreign key references
            r += ` REFERENCES ${parent} (${__setKeyColumns(parentColumns)})`;
            //
            // -- foreign key events
            if (onDelete || onUpdate) {
                const __setEvent = (ev, action) => {
                    if (['SET NULL', 'NO ACTION', 'RESTRICT', 'CASCADE', 'SET DEFAULT'].indexOf(action.toUpperCase()) > -1)
                        return `${(ev == 'delete' ? ' ON DELETE ' : ' ON UPDATE ')} ${action.toUpperCase()}`;
                };

                if (onDelete)
                    r += __setEvent('delete', onDelete);

                if (onUpdate)
                    r += __setEvent('update', onUpdate);
            }

            return r;

        } else
            throw new Error('Error in : ' + __filename + ' in function() x in const _set: __columns and __parentColumns must have same length');
    };


    if (Array.isArray(opt))
        return opt.map(exec).join();
    else if ("object" === typeof opt)
        return exec(opt);
};

/**
 * @param {string} tableName "name of table" 
 * @param {object} fkOpt
 * @prop {string} [name] "name of foreign key"
 * @prop {(string | string[])} columns "column name or list columns name of table"
 * @prop {string} parent "name of reference table"
 * @prop {(string|string[])} parentColumns "column name or list columns name of reference table"
 * @param {string} cmd "values could be (create|drop|alter)" 
 */
module.exports = (tableName, fkOpt, cmd) => {



    const __isOk = (function() {
        const __isArrayOk = ("array" === typeOf(fkOpt) && fkOpt.length > 0);
        const __isObjectOk = ("object" === typeOf(fkOpt) && Object.keys(fkOpt).length > 0);
        return (__isArrayOk || __isObjectOk);
    }());



    if (__isOk) {
        let r = `ALTER TABLE ${tableName} `;
        //
        const create = opt => {

            if (Array.isArray(opt)) {
                return opt.map(e => ` ADD ${_set(e)} `).join();
            } else if ("object" === typeof opt)
                return _set(opt);
        };
        //
        //
        const drop = opt => {
            if ("string" === typeof opt)
                return ` DROP FOREIGN KEY FOREIGN KEY ${opt}`;
            else if (Array.isArray(opt))
                return opt.map(e => ` DROP FOREIGN KEY ${e}`).join();
        };
        //
        //
        const alter = opt => {
            const to_drop = Array.isArray(opt) ? opt.map(e => e.name) : opt.name;
            return (` ${drop(to_drop)}, ${create(opt)} `);
        };


        if ("create" === cmd)
            return (r + create(fkOpt));
        else if ("drop" === cmd)
            return (r + drop(fkOpt));
        else if ("alter" === cmd)
            return (r + alter(fkOpt));
        else if ("get_create" === cmd)
            return create(fkOpt);
        else if ("get_drop" === cmd)
            return drop(fkOpt);
        else if ("get_alter" === cmd)
            return alter(fkOpt);
        else
            return _set(fkOpt);
    } else
        throw new Error('Error in ' + __filename);
};