/* jshint esversion : 6 */

const $app = require('app-code-js');
const { areEquals } = $app;
//
const queryComposer = require('./cmp');


class MySqlForeignKey extends require('../db-indexes/index') {

    constructor(opt) {
        super(opt);
    }


    setParentInnodDb(a) {
        const q = $q.defer();

        const queue = $q.chain();
        const fn = v => () => this.setEngine(v, 'InnoDB').then(q.resolve, q.reject);
        a.forEach(v => queue.push(fn(v)));

        queue.exec().then(q.resolve, q.reject);

        return q.promise;
    }


    showForeignKey(tableName, dbName) {
        const q = $q.defer();

        const query = '' +
            "SELECT TABLE_NAME AS 'onTable', COLUMN_NAME AS 'columns', CONSTRAINT_NAME AS name, REFERENCED_TABLE_NAME AS parent, REFERENCED_COLUMN_NAME AS parentColumns " +
            "FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE " +
            "WHERE TABLE_NAME = '" + tableName + "' AND TABLE_SCHEMA = '" + (dbName || this.dbName) + "' AND REFERENCED_TABLE_NAME IS NOT NULL";

        this.query(query).then(
            k => {
                const a = k.reduce((curr, next) => {

                    if (!curr.find(e => e.parent === next.parent)) {
                        curr.push({
                            onTable: next.onTable,
                            columns: [next.columns],
                            name: next.name,
                            parent: next.parent,
                            parentColumns: [next.parentColumns]
                        });
                    } else {

                        return curr.map(e => {
                            if (e.parent === next.parent) {
                                e.columns.push(next.columns);
                                e.parentColumns.push(next.parentColumns);
                            }
                            return e;
                        });

                    }

                    return curr;
                }, []);

                q.resolve(a);
            },
            q.reject
        );

        return q.promise;
    }

    showForeignKeyChildren(tableName, dbName) {

        const q = $q.defer();

        const query = "" +
            "SELECT  idx.*, CONSTRAINT_NAME, REFERENCED_TABLE_NAME,REFERENCED_COLUMN_NAME " +
            "FROM ( " +
            "    SELECT *, table_name as 'table', index_name as key_name " +
            "    FROM information_schema.statistics idx " +
            "    WHERE idx.TABLE_SCHEMA = '" + tableName + "'" +
            ") idx " +
            "INNER JOIN ( " +
            "    SELECT TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,CONSTRAINT_NAME, REFERENCED_TABLE_NAME,REFERENCED_COLUMN_NAME " +
            "    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE pk " +
            "    WHERE pk.TABLE_SCHEMA = '" + (dbName || this.dbName) + "' AND REFERENCED_TABLE_NAME = '" + tableName + "'" +
            ") fk " +
            "ON idx.TABLE_SCHEMA = fk.TABLE_SCHEMA AND fk.REFERENCED_TABLE_NAME <> idx.TABLE_NAME";

        this.query(query).then(r => q.resolve(r.groupBy('table')), q.reject);


        return q.promise;
    }




    createForeignKeyQuery(tableName, opt) {
        return queryComposer(tableName, opt, "get_create");
    }
    createForeignKey(tableName, opt, dbName) {

        const q = $q.defer();

        const parents = Object.keys(opt.groupBy('parent')).unique();



        this.setParentInnodDb(parents).then(
            () => this.query(queryComposer(tableName, opt, "create")).then(q.resolve, q.reject),
            q.reject
        );

        return q.promise;
    }


    alterForeignKeyQuery(tableName, opt) {
        return queryComposer(tableName, opt, "get_alter");
    }
    alterForeignKey(tableName, opt, dbName) {



        const q = $q.defer();
        const parents = Object.keys(opt.groupBy('parent')).unique();

        this.setParentInnodDb(parents).then(
            () => {
                const _tableName = this.db_dot_table(tableName, dbName);
                const toDrop = opt.map(e => e.name);

                const dropQuery = queryComposer(_tableName, toDrop, "drop");
                const createQuery = queryComposer(_tableName, opt, "create");

                this.query(dropQuery).then(
                    () => this.query(createQuery).then(q.resolve, q.reject),
                    q.reject
                );

            },
            q.reject
        );


        return q.promise;
    }


    dropForeignKeyQuery(tableName, opt) {
        return queryComposer(tableName, opt, "get_drop");
    }
    dropForeignKey(tableName, opt, dbName) {
        return this.query(queryComposer(this.db_dot_table(tableName, dbName), opt, "drop"));
    }



    getForeignKeyOptions(tableName, opt, dbName) {

        const q = $q.defer();

        this.showForeignKey(tableName, dbName).then(
            fkList => {



                const create = [];
                const alter = [];


                const checkFk = o => {
                    //
                    const _stringifyColumns = c => Array.isArray(c) ? c.join() : c;
                    const fk = fkList.find(e => e.name === o.name);

                    let { columns, parentColumns } = o;
                    columns = _stringifyColumns(columns);
                    parentColumns = _stringifyColumns(parentColumns);


                    if (!fk)
                        create.push(o);
                    else if (!areEquals(columns, _stringifyColumns(fk.columns)) || !areEquals(parentColumns, _stringifyColumns(fk.parentColumns)))
                        alter.push(o);
                };


                if (!Array.isArray(opt) && "object" === typeof opt)
                    opt = [opt];


                opt.forEach(v => checkFk(v));


                q.resolve({
                    create: create,
                    alter: alter,
                    drop: fkList.filter(e => {
                        if (!opt.find(o => (o.name === e.name)))
                            return e;
                    }).map(e => e.name)
                });

            },
            q.reject
        );


        return q.promise;
    }





    foreignKey(tableName, opt, dbName) {

        const q = $q.defer();


        this.getForeignKeyOptions(tableName, opt, dbName).then(
            r => {

                const execFn = (a, cmd) => () => {
                    const q = $q.defer();
                    if (!a.length)
                        q.resolve();
                    else {
                        this[cmd](tableName, a, dbName).then(q.resolve, q.reject);
                    }

                    return q.promise;
                };



                const _qChain = $q.chain(),
                    _queue = [
                        execFn(r.drop, 'dropForeignKey'),
                        execFn(r.alter, 'alterForeignKey'),
                        execFn(r.create, 'createForeignKey')
                    ];

                _qChain.push.apply(_qChain, _queue).exec().then(q.resolve, q.reject);

            },
            q.reject
        );



        return q.promise;
    }


}

module.exports = MySqlForeignKey;