/* jshint esversion : 6 */
const { areEquals, typeOf } = require('app-code-js');



const _set = opt => {

    const exec = ({ unique, name, columns, indexType, keyBlockSize, parser, comment }) => {
        //
        // - unique or not
        let r = (unique ? 'UNIQUE INDEX ' : ' INDEX ');
        //        
        // -- index name
        r += ((name || "") + ' (');
        //
        // -- index columns
        if ("string" === typeof columns)
            r += (columns + ')');
        else if (Array.isArray(columns))
            r += (columns.join() + ')');
        //
        // -- index type
        if (indexType)
            r += ` USING ${indexType} `;
        //
        // -- index key_block_size
        if (keyBlockSize)
            r += `KEY_BLOCK_SIZE = ${keyBlockSize} `;
        //
        // -- use parser
        if (parser)
            r += ` WITH PARSER ${parser} `;
        //
        // -- add comment to index
        if (comment)
            r += ` COMMENT ${comment} `;

        return r;
    };


    if (Array.isArray(opt))
        return opt.map(exec).join();
    else if ("object" === typeof opt)
        return exec(opt);
};


/**
 * @param {string} tableName "name of table" 
 * @param {object} indexOpt
 * @prop {string} [name] "name of index"
 * @prop {boolean} [unique] "index unique or non_unique"
 * @prop {(string|string[])} [columns] "columns of index"
 * @param {string} cmd "values could be (create|drop|alter)" 
 */
module.exports = (tableName, indexOpt, cmd) => {



    const __isOk = (function() {
        const __isArrayOk = ("array" === typeOf(indexOpt) && indexOpt.length > 0);
        const __isObjectOk = ("object" === typeOf(indexOpt) && Object.keys(indexOpt).length > 0);
        return (__isArrayOk || __isObjectOk);
    }());



    if (__isOk) {

        let r = `ALTER TABLE ${tableName} `;
        //
        const create = opt => {
            if (Array.isArray(opt)) {
                return opt.map(e => ` ADD ${_set(e)} `).join();
            } else if ("object" === typeof opt)
                return createIndex(opt);
        };
        //
        const drop = opt => {
            if ("string" === typeof opt)
                return ` DROP INDEX ${opt} `;
            else if (Array.isArray(opt))
                return opt.map(e => ` DROP INDEX ${e} `).join();
        };
        //
        const alter = opt => {
            const to_drop = (Array.isArray(opt) ? opt.map(e => e.name) : opt.name);
            return ` ${drop(to_drop)}, ${create(opt)}`;
        };




        if ("create" === cmd)
            return (r + create(indexOpt));
        else if ("drop" === cmd)
            return (r + drop(indexOpt));
        else if ("alter" === cmd)
            return (r + alter(indexOpt));
        else if ("get_create" === cmd)
            return create(indexOpt);
        else if ("get_drop" === cmd)
            return drop(indexOpt);
        else if ("get_alter" === cmd)
            return alter(indexOpt);
        else
            return _set(indexOpt);
    }
};