/* jshint esversion : 6 */
const $app = require('app-code-js');
const { areEquals } = $app;

const where = require('../db-queries/where');
const cmpIndex = require('./cmp');



const mapIdxFn = e => {
    e.toLowerKeys();
    if (e.column_name)
        e.name = e.column_name;

    return e;
};


class MySqlIndexes extends require('../db-queries/index') {
    constructor(opt) {
        super(opt);
    }

    showIndex(tableName, o) {

        const { db: dbName } = (o || {});

        const q = $q.defer();

        let query = "" +
            "SELECT * FROM( " +
            "    SELECT A.TABLE_SCHEMA,  A.COLUMN_NAME, KEY_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, NULLABLE, SUB_PART, PACKED, SEQ_IN_INDEX,  CARDINALITY,COLLATION FROM( " +
            "        SELECT * FROM( " +
            "            SELECT idx.*, PARENT_OF_TABLE FROM ( " +
            "                SELECT  TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME, REFERENCED_TABLE_NAME , REFERENCED_COLUMN_NAME  " +
            "                FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE pk " +
            "            ) idx " +
            "            LEFT JOIN ( " +
            "                SELECT  TABLE_SCHEMA,TABLE_NAME, REFERENCED_TABLE_NAME as PARENT_OF_TABLE, REFERENCED_COLUMN_NAME as COLUMN_NAME " +
            "                FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE pk " +
            "            ) pk " +
            "            ON idx.TABLE_SCHEMA = pk.TABLE_SCHEMA AND idx.TABLE_NAME = pk.PARENT_OF_TABLE " +
            "        ) idx " +
            "        WHERE TABLE_SCHEMA = '" + (dbName || this.dbName) + "' AND (TABLE_NAME = '" + tableName + "' OR REFERENCED_TABLE_NAME = '" + tableName + "' OR PARENT_OF_TABLE = '" + tableName + "') " +
            "   ) A " +
            "   RIGHT JOIN ( " +
            "       SELECT  *,  index_name as key_name " +
            "       FROM information_schema.statistics idx " +
            "       WHERE idx.TABLE_SCHEMA ='" + (dbName || this.dbName) + "' AND idx.TABLE_NAME ='" + tableName + "'" +
            "   ) B " +
            "   ON A.TABLE_SCHEMA = B.TABLE_SCHEMA AND A.COLUMN_NAME = B.COLUMN_NAME " +
            "   ORDER BY REFERENCED_TABLE_NAME DESC " +
            "   LIMIT 500 " +
            ") idx " +
            "GROUP BY TABLE_SCHEMA, COLUMN_NAME, KEY_NAME";


        this.query(query).then(r => {
            q.resolve(r.map(mapIdxFn));
        }, q.reject);


        return q.promise;
    }

    showNonUniqueIndex(tableName, dbName) {
        return this.query(` SHOW INDEX FROM ${this.db_dot_table(tableName, dbName)}  WHERE NON_UNIQUE = 1 `);
    }

    showUniqueIndex(tableName, dbName) {
        return this.query(` SHOW INDEX FROM ${this.db_dot_table(tableName, dbName)}  WHERE NON_UNIQUE = 0 `);
    }

    groupIndexByName(tableName, dbName) {
        const q = $q.defer();

        this.showIndex(tableName, dbName).then(
            r => {

                const tableIndexes = r.groupBy('key_name', true)
                    .map(e => {
                        const col = e.map(e => e.column_name).unique().filter(e => {
                            if (e != null && "undefined" !== typeof e)
                                return e;
                        });



                        return e.map(x => {
                            return {
                                name: x.key_name,
                                unique: !x.non_unique ? 1 : 0,
                                columns: col,
                                primary: x.key_name.match(/primary/i) != null,
                                foreignKey: (x.referenced_table_name != null && x.referenced_column_name != null)
                            };
                        }).unique()[0];

                    });


                q.resolve(tableIndexes);

            }, q.reject);

        return q.promise;
    }

    /**
     * @param {string} tableName "table name"
     * @param { {name:string, unique:boolean, columns:(string|sting[])} } opt "index options" 
     * @prop {string} name "name of index"
     * @prop {boolean} unique "unique index"
     * @prop {(string|string[])} columns "column name or list columns name of index"
     * @param {string} [dbName] "db name"
     */
    createIndexQuery(tableName, opt, dbName) {
        return cmpIndex(this.db_dot_table(tableName, dbName), opt, 'get_create');
    }
    createIndex(tableName, opt, dbName) {
        const query = cmpIndex(this.db_dot_table(tableName, dbName), opt, 'create');
        return this.query(query);
    }

    /**
     * @param {string} tableName "table name"
     * @param {object} opt      
     * @prop {(string|sring[])} drop "index name or list of indexes name to drop"
     * @param {tring} [dbName] "db name"
     */
    dropIndexQuery(tableName, opt, dbName) {
        return cmpIndex(this.db_dot_table(tableName, dbName), opt, 'get_drop');
    }
    dropIndex(tableName, opt, dbName) {
        const query = cmpIndex(this.db_dot_table(tableName, dbName), opt, 'drop');
        return this.query(query);
    }

    /**
     * @param {string} tableName "table name"
     * @param { {name:string, unique:boolean, columns:(string|sting[])} } opt "index options" 
     * @prop {string} name "name of index"
     * @prop {boolean} unique "unique index"
     * @prop {(string|string[])} columns "column name or list columns name of index"
     * @param {string} [dbName] "db name"
     */
    alterIndexQuery(tableName, opt, dbName) {
        return cmpIndex(this.db_dot_table(tableName, dbName), opt, 'get_alter');
    }
    alterIndex(tableName, opt, dbName) {
        const query = cmpIndex(this.db_dot_table(tableName, dbName), opt, 'alter');
        return this.query(query);
    }


    getIndexOptions(tableName, opt, dbName) {

        const q = $q.defer();

        this.groupIndexByName(tableName, dbName).then(
            indexes => {

                const _tableIndexes = indexes.filter(e => {
                    if (!e.foreignKey)
                        return e;
                }).map(e => e.name);


                const create = [];
                const alter = [];


                const checkIndexes = o => {
                    const index = indexes.find(e => e.name === o.name);


                    o.unique = !o.unique ? 0 : 1;

                    const _stringifyColumns = c => Array.isArray(c) ? c.join() : c;

                    if (!index)
                        create.push(o);
                    else if (_tableIndexes.indexOf(o.name) > -1 && (index.unique !== o.unique || !areEquals(_stringifyColumns(o.columns), _stringifyColumns(index.columns))))
                        alter.push(o);

                };

                if (!Array.isArray(opt) && "object" === typeof opt)
                    opt = [opt];



                if (Array.isArray(opt)) {
                    opt.forEach(v => checkIndexes(v));


                    q.resolve({
                        create: create,
                        alter: alter,
                        drop: indexes.filter(e => {
                            if (!e.name.match(/^primary$/i) && !e.primary && !e.foreignKey && !opt.find(o => o.name == e.name))
                                return e;
                        }).map(e => e.name)
                    });

                } else
                    q.reject('you must pass an array or an object in parameter 2');

            },
            q.reject
        );


        return q.promise;
    }


    index(tableName, opt, dbName) {

        const q = $q.defer();


        this.getIndexOptions(tableName, opt, dbName).then(
            r => {



                const execFn = (a, cmd) => {
                    const q = $q.defer();
                    if (!a.length)
                        q.resolve();
                    else {
                        this[cmd](tableName, a, dbName).then(q.resolve, q.reject);
                    }

                    return q.promise;
                };

                const { drop, alter, create } = r;



                const _qChain = $q.chain(),
                    _queue = [
                        execFn(drop, 'dropIndex'),
                        execFn(alter, 'alterIndex'),
                        execFn(create, 'createIndex')
                    ];

                _qChain.push.apply(_qChain, _queue).exec().then(q.resolve, q.reject);

            },
            q.reject
        );



        return q.promise;
    }

}

module.exports = MySqlIndexes;