/* jshint esversion : 6 */

require('app-code-js');

const createTable = require('./cmp');


class MySqlTableManager extends require('../db-columns/index') {
    constructor(opt) {
        super(opt);
    }

    selectTableSchema(tableName) {
        return this.query(` SELECT * FROM information_schema.tables WHERE table_schema = '${this.dbName}' AND table_name = '${tableName}' `);
    }


    existTableSchema(tableName) {
        const q = $q.defer();

        this.selectTableSchema(tableName).then(r => q.resolve(r && r.length > 0), q.reject);

        return q.promise;
    }


    showEngine(tableName, dbName) {
        const q = $q.defer();

        const query = ` SELECT ENGINE FROM information_schema.TABLES WHERE TABLE_SCHEMA='${(dbName || this.dbName)}' AND TABLE_NAME = '${tableName}' `;
        this.query(query).then(r => q.resolve(r.length ? r[0].ENGINE : "none"), q.reject);

        return q.promise;
    }


    setEngine(tableName, engine) {
        return this.query(` ALTER TABLE ${tableName} ENGINE=${engine} `);
    }

    createTable(tableName, opt, dbName) {
        const q = $q.defer();



        const setInnodDbFk = () => {
            const q = $q.defer();

            let { foreignKey } = opt;


            if (!foreignKey)
                q.resolve();
            else {

                if (typeof foreignKey === "object" && !Array.isArray(foreignKey))
                    foreignKey = [foreignKey];

                const parents = Object.keys(foreignKey.groupBy('parent')).unique();

                this.setParentInnodDb(parents).then(q.resolve, q.reject);
            }

            return q.promise;
        };

        setInnodDbFk().then(
            () => {
                const _tableName = this.db_dot_table(tableName, dbName);
                this.query(createTable(_tableName, opt)).then(q.resolve, q.reject);
            },
            q.reject
        );

        return q.promise;
    }



    alterTable(tableName, opt, dbName) {

        const exec_fn = (n, opt) => () => {

            const q = $q.defer();

            if (!opt)
                q.resolve();
            else {
                this[n](tableName, opt).then(q.resolve, q.reject);
            }
            return q.promise;
        };

        const queue = $q.chain();

        const _exec = ['columns', 'index', 'foreignKey'].map(e => exec_fn(e, opt[e]));

        return queue.push.apply(queue, _exec).exec();
    }




    setTable(tableName, opt) {


        const q = $q.defer();


        this.existTableSchema(tableName).then(
            exist => {


                const create = opt => {
                    const q = $q.defer();

                    if (!exist) {
                        this.createTable(tableName, opt).then(q.resolve, q.reject);
                    } else
                        q.resolve(exist);

                    return q.promise;
                };


                create(opt).then(
                    () => this.alterTable(tableName, opt).then(q.resolve, q.reject),
                    err => q.reject(err)
                );

            },
            q.reject
        );


        return q.promise;
    }

    /**
     * @param {(string|object)} newTableOpt "if it's an object must have @prop {string} name 'table name' and   @prop {string} db 'db name'"
     * @param {(string|object)} oldTableOpt "if it's an object must have @prop {string} name 'table name' and   @prop {string} db 'db name'"
     * @param {boolean} [includeIdx=null] 
     */
    cloneTable(newTableOpt, oldTableOpt, includeIdx) {

        const opt = a => {
            if ("string" === typeof a) {
                return this.db_dot_table(a);
            } else if ("object" === typeof a) {
                return this.db_dot_table(a.name, a.db);
            }
        };

        const newTableName = opt(newTableOpt);
        const oldTableName = opt(oldTableOpt);

        const q = $q.defer();

        if (!includeIdx) {
            this.query(` CREATE TABLE ${newTableName} AS SELECT * FROM ${oldTableName} `).then(q.resolve, q.reject);

        } else {
            this.query(` CREATE TABLE ${newTableName} LIKE ${oldTableName} `)
                .then(
                    () => this.query(` INSERT TABLE ${newTableName} AS SELECT * FROM ${oldTableName} `).then(q.resolve, q.reject),
                    q.reject
                );
        }


        return q.promise;
    }

}

module.exports = MySqlTableManager;