/* jshint esversion : 6 */

require('app-code-js');


class TableSchemaModel {
    constructor(tableName, opt, db) {
        this.tableName = tableName;
        this._db = db;
        this._option = opt;

        let id_field = null;
        if (opt && opt.columns && opt.columns.length) {
            const id_f = opt.columns.filter(e => {
                if (e.autoincrement || e.primaryKey)
                    return e;
            }).map(e => e.name);

            if (id_f.lenght)
                id_field = id_f[0];
        }

        this._id_field = id_field;


        this._ready = $q.defer();
    }

    get ready() {
        return this._ready.promise;
    }

    copyTableInto(newTableOpt, includeIdx) {
        return this._db.cloneTable(newTableOpt, this.tableName, includeIdx);
    }

    createTableLike(oldTableOpt, includeIdx) {
        return this._db.cloneTable(this.tableName, oldTableOpt, includeIdx);
    }

    init() {

        $q.when(this._db.ready, () => {
            if (!this._option)
                this._readys.resolve();
            else {
                this._db.setTable(this.tableName, this._option).then(this._ready.resolve, this._ready.reject);
            }
        });
        return this._ready.promise;
    }

}


class TableSchemaQuery extends TableSchemaModel {
    constructor(tableName, opt, db) {
        super(tableName, opt, db);
    }

    query(...a) {
        return this._db.query.apply(this._db, a);
    }

    find(opt) {
        return this._db.find(this.tableName, opt);
    }

    watchQueryResult(queryOpt, fn, opt) {
        return this._db.watchQueryResult(this.tableName, queryOpt, fn, opt);
    }

    put(a) {
        return this._db.put(this.tableName, a);
    }

    insert(a) {
        return this._db.insert(this.tableName, a);
    }

    update(a) {
        return this._db.update(this.tableName, a);
    }

    delete(a) {
        return this._db.delete(this.tableName, a);
    }
}




class TableSchemaIndex extends TableSchemaQuery {
    constructor(tableName, opt, db) {
        super(tableName, opt, db);
    }

    showIndex() {
        return this._db.showIndex(this.tableName);
    }

    showNonUniqueIndex() {
        return this._db.showNonUniqueIndex(this.tableName);
    }

    showUniqueIndex() {
        return this._db.showUniqueIndex(this.tableName);
    }

    groupIndexByName() {
        return this._db.groupIndexByName(this.tableName);
    }

    createIndex(opt) {
        return this._db.createIndex(this.tableName, opt);
    }

    dropIndex(opt) {
        return this._db.dropIndex(this.tableName, opt);
    }

    alterIndex(opt) {
        return this._db.alterIndex(this.tableName, opt);
    }

    index(opt) {
        return this._db.index(this.tableName, opt);
    }
}



class TableSchemaFk extends TableSchemaIndex {
    constructor(tableName, opt, db) {
        super(tableName, opt, db);
    }
}


class TableSchemaColumns extends TableSchemaFk {
    constructor(tableName, opt, db) {
        super(tableName, opt, db);
    }


    showColumns() {
        return this._db.showColumns();
    }

    addColumns(opt) {
        return this._db.addColumns(this.tableName, opt);
    }


    dropColumns(opt) {
        return this._db.dropColumns(this.tableName, opt);
    }


    modifyColumns(opt) {
        return this._db.modifyColumns(this.tableName, opt);
    }


    columns(opt) {
        return this._db.columns(this.tableName, opt);
    }
}



class TableSchema extends TableSchemaColumns {
    constructor(tableName, opt, db) {
        super(tableName, opt, db);
    }

    showForeignKey() {
        return this._db.showForeignKey(this.tableName);
    }

    createForeignKey(opt) {
        return this._db.createForeignKey(this.tableName, opt);
    }

    alterForeignKey(opt) {
        return this._db.alterForeignKey(this.tableName, opt);
    }

    dropForeignKey(opt) {
        return this._db.dropForeignKey(this.tableName, opt);
    }
}


module.exports = (tableName, opt, db) => new TableSchema(tableName, opt, db);